-- Local variables
local countStep = 0
local class, race = "", ""
local db = {}
local GUI = {}
local sName, sTexture, sLevel = "", "", 0
local L = ProGuideLocals
local ActiveObj = false
-- returns a table uniqe number based on the table size with all sub tables
local function NumOfSteps(guide)
  if not guide or type(guide) ~= "table" then return 0 end

  countStep = 0
  for part, info in ipairs(guide.Part) do
    for step, value in ipairs(info.Step) do
      countStep = countStep + 1
    end
  end

  return countStep
end

-- Class Check
local function isClass(check)
  class = UnitClassBase("player")

  if type(check) == "string" and check == class then
    return true
  elseif type(check) == "table" and tContains(check, class) then
    return true
  else
    return false
  end
end

-- Race Check
local function isRace(check)
  race = UnitRace("player")

  if type(check) == "string" and check == race then
    return true
  elseif type(check) == "table" and tContains(check, race) then
    return true
  else
    return false
  end
end

-- Activate Guide if possible
local function ActivateGuide(gname)
  if ProGuide.db then
    db = ProGuide.db.char.Guides
    if ProGuide.Guides[gname]["TotalSteps"] == db[gname]["TotalSteps"] then
      ProGuide.Guides[gname]["Active"] = true
    else
      ProGuide.Guides[gname]["Active"] = false
    end
  else
    ProGuide.Guides[gname]["Active"] = false
  end
end

-- Get Current Part of guide
local function GetActivePartStep(guide)
  for part, info in ipairs(guide.Part) do
    if not info.Done then 
      for step, value in ipairs(guide.Part[part].Step) do
        if not value.Done then
          return part, step
        end
      end
      -- cleanup part since all step was done but part was not flaged done
      guide.Part[part].Step[step].Done = true
    end
  end
  -- if we reach this, then the guide is done and we need to tell that to the handler
  return 0,0
end

-- add guide to character guide progress
local function AddGuideToCharGuideDB(gname, guide)
  if not gname or type(gname) ~= "string" then return end
  if not guide or type(guide) ~= "table" then return end

  db = ProGuide.db.char.Guides
  -- only add/update if not exist or TotalSteps not the same
  if (not db[gname])
  or (not db[gname]["TotalSteps"] or db[gname]["TotalSteps"] ~= NumOfSteps(guide))
  then
    -- reset guide
    db[gname] = {}
    db[gname]["TotalSteps"] = NumOfSteps(guide)
    for part, info in ipairs(guide.Part) do
      -- add part status
      if not db[gname]["Part"] then db[gname]["Part"] = {} end
      db[gname]["Part"][part] = {}
      db[gname]["Part"][part]["Done"] = false
      for step, value in ipairs(info.Step) do
        -- add step status
        if not db[gname]["Part"][part]["Step"] then db[gname]["Part"][part]["Step"] = {} end
        db[gname]["Part"][part]["Step"][step] = {}
        db[gname]["Part"][part]["Step"][step]["Done"] = false
      end
    end
    return true
  else
    return false
  end
end

-- Cleanup temp DB
function ProGuide:CleanUpTempDB(tempDB)
  for gname, guide in pairs(tempDB) do
    AddGuideToCharGuideDB(gname, guide)
    ActivateGuide(gname)
  end
end

-- Add data to database
function ProGuide:AddData(lang, data)
  -- Check if valid data have been added
  if not lang or lang ~= GetLocale() then return end
  if not data or type(data) ~= "table" then return end

  -- Add new data to Data database.
  for group,table in pairs(data) do
    -- Create the groupe if not there
    if not ProGuide.Data[group] then ProGuide.Data[group] = {} end

    for key, value in pairs(table) do
      -- add values if not found
      if not ProGuide.Data[group][key] then
        ProGuide.Data[group][key] = value
      -- update values if mustupdate value is presented
      elseif value.mustupdate then
        ProGuide.Data[group][key] = value
      end
    end
  end
end

-- Add guide to database
function ProGuide:AddGuide(lang, fac, gname, guide)
  if not lang then lang = GetLocale() end
  if lang ~= GetLocale() then return end
  if not fac then fac = UnitFactionGroup("player") end
  if fac ~= UnitFactionGroup("player") then return end
  if not gname then return end
  if not guide or type(guide) ~= "table" then return end
  -- Check if player is the correct class if set
  if guide.Class and (not isClass(guide.Class)) then return end
  -- Check if player is the correct race if set
  if guide.Race and (not isRace(guide.Race)) then return end

  -- Manage DB info
  if not ProGuide.db then
    if not ProGuide.TempGuideDB[gname] then ProGuide.TempGuideDB[gname] = guide end
  else
    AddGuideToCharGuideDB(gname, guide)
  end

  -- Check if there is one with the same name and then skip it so we don't overwrite it
  if ProGuide.Guides[gname] then return end

  ProGuide.Guides[gname] = guide
  ProGuide.Guides[gname]["TotalSteps"] = NumOfSteps(guide)

  -- Activate guide if all is ready
  ActivateGuide(gname)
end

-- Load a guide into the Guide Frame
function ProGuide:LoadGuide(Obj, GuideName)
  if ProGuide.Guides[GuideName] and ProGuide.Guides[GuideName]["Active"] then
    db = ProGuide.db.char.Guides[GuideName]
    
    -- get what part and step we are at
    db["ActivePart"], db["ActiveStep"] = GetActivePartStep(db)
    
    -- if guide is done, then we will not load this guide
    if db["ActivePart"] == 0 then return end
    
    -- make shortcut for the guide
    GUI = ProGuide.Guides[GuideName]
    
    -- Add Icon if one is set
    if GUI.Icon then ProGuide:EditIcon(Obj, GUI.Icon) end
    
    -- Enable Buy List if set
    if GUI.Part[db["ActivePart"]].UseBuyList then
      if not Obj.ButtonBuyList:IsEnabled() then Obj.ButtonBuyList:Enable() end
      Obj.BuyList.Part = db["ActivePart"]
    else
      if Obj.ButtonBuyList:IsEnabled() then Obj.ButtonBuyList:Disable() end
    end
    
    -- Add the Guide name to the objective frame
    ProGuide:EditHeadline(Obj, GUI.Name)
    
    -- Add current skill level if specified
    if GUI.ShowSkillLevel then ProGuide:SetSkillLevel(Obj, GUI.ShowSkillLevel) end

    -- Add Sub Title
    ProGuide:EditSubHeadline(Obj, GUI.Part[db["ActivePart"]].Title)
    
    -- Add Objectives
    for step, info in ipairs(GUI.Part[db["ActivePart"]].Step) do
      if step == db["ActiveStep"] then ActiveObj = true else ActiveObj = false end
      -- Add Train Skill to objective
      if info.TrainSkill then
        ProGuide:AddTrainSkill(Obj, ActiveObj, info.TrainSkill, db.Part[db.ActivePart].Step[step].Done)
      end

      -- Add Learn Item to objective
      if info.LearnItem then
        ProGuide:AddLearnItem(Obj, ActiveObj, info.LearnItem, db.Part[db.ActivePart].Step[step].Done)
      end

      -- Add Create to objective
      if info.Create then
        ProGuide:AddCreateItem(Obj, ActiveObj, info.Create, db.Part[db.ActivePart].Step[step].Done)
      end

      -- Add Level Skill to objective
      if info.LevelSkill then
        ProGuide:AddLevelSkill(Obj, ActiveObj, info.LevelSkill, db.Part[db.ActivePart].Step[step].Done)
      end

      --print(step)
    end
  end
end